package com.winhc.bigdata.spark.jobs.chance

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, ReflectUtils, SparkUtils}
import org.apache.spark.internal.Logging
import org.apache.spark.sql.functions.col
import org.apache.spark.sql.types.{MapType, StringType, StructField, StructType}
import org.apache.spark.sql.{DataFrame, Row, SparkSession}

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/7/7 11:25
 * @Description: 筛选出数据的具体变更
 */
object ChangeExtract {

  //判断两个map在指定key上是否相等，如不等反回不相等字段
  def getDoubleDataMap(iterable: Iterable[Map[String, String]]): (Map[String, String], Map[String, String]) = {
    val map = iterable.map(m => (m("change_flag"), m)).toMap
    (map("0"), map("1"))
  }

  def getHandleClazz(tableName: String, equCols: Seq[String]): {def handle(rowkey: String, oldMap: Map[String, String], newMap: Map[String, String]): (String, String, String, Map[String, String], String, String, String, String, Map[String, String])} = {
    val clazz = s"com.winhc.bigdata.spark.jobs.chance.$tableName"
    val foo = Class.forName(clazz)
      .getConstructors.head.newInstance(equCols)
      .asInstanceOf[ {
      def handle(rowkey: String, oldMap: Map[String, String], newMap: Map[String, String]): (String, String, String, Map[String, String], String, String, String, String, Map[String, String])
    }]
    foo
  }


  case class ChangeExtractHandle(s: SparkSession,
                                 project: String, //表所在工程名
                                 tableName: String, //表名（不加前后辍）
                                 primaryKey: String, //此维度主键
                                 inc_ds: String, //需要计算的分区
                                 primaryFields: Seq[String] //主要字段，该字段任意一个不同 则认为发生变化
                                ) extends LoggingUtils with Logging {
    @(transient@getter) val spark: SparkSession = s


    val target_eci_change_extract = "ads_change_extract"

    def calc(isCopy: Boolean = true): Unit = {
      val cols = primaryFields.filter(!_.equals(primaryKey)).seq

      val ds = inc_ds.replace("-", "")

      val intersectCols = getColumns(s"$project.ads_$tableName").toSet & getColumns(s"$project.inc_ads_$tableName").toSet

      val otherAllCols = intersectCols.filter(!primaryKey.equals(_)).toSeq
      val all_cols = primaryKey +: otherAllCols :+ "change_flag"

      val lastDs_ads_all = getLastPartitionsOrElse(s"$project.ads_$tableName", "0")

      val handle = ReflectUtils.getClazz[CompanyChangeHandle](s"com.winhc.bigdata.spark.jobs.chance.table.$tableName", cols)
      //      val handle = getHandleClazz(tableName, cols)

      val update_time = BaseUtil.nowDate()

      var df: DataFrame = null
      isCopy match {
        case true => {
          sql(
            s"""
               |SELECT  cid,current_cid as new_cid
               |FROM    ${project}.inc_ods_company
               |WHERE   ds > $lastDs_ads_all and ds < $ds
               |AND     cid IS NOT NULL
               |AND     current_cid IS NOT NULL
               |GROUP BY cid,current_cid
               |""".stripMargin).createOrReplaceTempView("mapping")

          val cid = getColumns(s"$project.ads_$tableName").filter(f => f.equals("cid") || f.equals("new_cid")).max
          df = sql(
            s"""
               |SELECT  $primaryKey,${otherAllCols.mkString(",")},'0' as change_flag
               |FROM    $project.inc_ads_$tableName
               |WHERE   ds = $ds
               |UNION ALL
               |SELECT  t2.$primaryKey,${otherAllCols.map("t2." + _).mkString(",")},'1' as change_flag
               |FROM    (
               |            SELECT  DISTINCT ${primaryKey}
               |            FROM    $project.inc_ads_$tableName
               |            WHERE   ds = $ds
               |        ) AS t1
               |JOIN    (
               |             SELECT  concat_ws('_',coalesce(mm.new_cid,tmp.$cid),split(rowkey, '_')[1]) AS rowkey
               |                     ,${intersectCols.diff(Set("rowkey", "cid", "new_cid")).mkString(",")}
               |                     ,coalesce(mm.new_cid,tmp.$cid) AS new_cid
               |                     ,tmp.$cid as cid
               |                     ,c
               |             FROM    (
               |                         SELECT  a.*
               |                                 ,row_number() OVER (PARTITION BY a.${primaryKey} ORDER BY update_time DESC) c
               |                         FROM    (
               |                                     SELECT  ${intersectCols.mkString(",")}
               |                                     FROM    $project.ads_$tableName
               |                                     WHERE   ds = $lastDs_ads_all
               |                                     UNION ALL
               |                                     SELECT  ${intersectCols.mkString(",")}
               |                                     FROM    $project.inc_ads_$tableName
               |                                     WHERE   ds > $lastDs_ads_all and ds < $ds
               |                                 ) AS a
               |                     ) AS tmp
               |             LEFT JOIN mapping mm
               |             ON tmp.$cid = mm.cid
               |             WHERE   tmp.c = 1
               |        ) AS t2
               |ON      t1.${primaryKey} = t2.${primaryKey}
               |""".stripMargin)
        }
        case false => {
          df = sql(
            s"""
               |SELECT  $primaryKey,${otherAllCols.mkString(",")},'0' as change_flag
               |FROM    $project.inc_ads_$tableName
               |WHERE   ds = $ds
               |UNION ALL
               |SELECT  t2.$primaryKey,${otherAllCols.map("t2." + _).mkString(",")},'1' as change_flag
               |FROM    (
               |            SELECT  DISTINCT ${primaryKey}
               |            FROM    $project.inc_ads_$tableName
               |            WHERE   ds = $ds
               |        ) AS t1
               |JOIN    (
               |             SELECT  tmp.*
               |             FROM    (
               |                         SELECT  a.*
               |                                 ,row_number() OVER (PARTITION BY a.${primaryKey} ORDER BY update_time DESC) c
               |                         FROM    (
               |                                     SELECT  ${intersectCols.mkString(",")}
               |                                     FROM    $project.ads_$tableName
               |                                     WHERE   ds = $lastDs_ads_all
               |                                     UNION ALL
               |                                     SELECT  ${intersectCols.mkString(",")}
               |                                     FROM    $project.inc_ads_$tableName
               |                                     WHERE   ds > $lastDs_ads_all and ds < $ds
               |                                 ) AS a
               |                     ) AS tmp
               |             WHERE   tmp.c = 1
               |        ) AS t2
               |ON      t1.${primaryKey} = t2.${primaryKey}
               |""".stripMargin)
        }
      }


      val rdd =
        df.select(all_cols.map(column => col(column).cast("string")): _*)
          .rdd.map(r => {
          (r.getAs[String](primaryKey), all_cols.map(f => (f, r.getAs[String](f))).toMap)
        }).groupByKey()
          .map(x => {
            val rowkey = x._1
            val map_list = x._2
            //          try {
            if (map_list.size == 1) {
              val res = handle.handle(rowkey, null, map_list.head)
              Row(res._1, res._2, tableName, res._3, res._4, res._5, res._6, res._7, res._8, update_time, res._9)
            } else {
              if (map_list.size > 2) {
                logInfo("list.size > 2! rowkey:" + rowkey)
              }
              val m = getDoubleDataMap(map_list)

              val new_map = m._1
              val old_map = m._2
              val res = handle.handle(rowkey, old_map, new_map)
              if (res == null) {
                null
              } else {
                Row(res._1, res._2, tableName, res._3, res._4, res._5, res._6, res._7, res._8, update_time, res._9)
              }
            }
            /* } catch {
               case e: Exception => {
                 logError(s"xjk rowkey:$rowkey msg:${e.getMessage} equCols:$cols")
                 logError(e.getMessage, e)
                 println(s"xjk rowkey:$rowkey msg:${e.getMessage} equCols:$cols")
               }
                 null
             }*/
          }).filter(_ != null)

      val schema = StructType(Array(
        StructField("rowkey", StringType), //表数据主建
        StructField("cid", StringType), //公司id
        StructField("table_name", StringType), //表名
        StructField("type", StringType), // 变更类型 insert update
        StructField("data", MapType(StringType, StringType)), //变更后数据
        StructField("fields", StringType), //如果是更新 则显示更新字段
        StructField("title", StringType), // 动态数据展示 ps. 新增某土地公示
        StructField("label", StringType), // 1.一般变更，2.风险变更
        StructField("biz_time", StringType), //业务时间
        StructField("update_time", StringType), //处理时间
        StructField("old_data", MapType(StringType, StringType)) //变更前数据
      ))

      spark.createDataFrame(rdd, schema)
        .createOrReplaceTempView("tmp_change_extract_view") //

      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE ${project}.$target_eci_change_extract PARTITION(ds='$ds',tn='$tableName')
           |SELECT *
           |FROM
           |    tmp_change_extract_view
           |""".stripMargin)
    }
  }


  // winhc_eci_dev company_tm rowkey 20200717 status_new
  // winhc_eci_dev company_patent_list rowkey 20200717 lprs
  // winhc_eci_dev company_certificate rowkey 20200707 type
  // winhc_eci_dev company_copyright_works_list rowkey 20200717 type
  // winhc_eci_dev company_copyright_reg_list rowkey 20200717 version
  // winhc_eci_dev company_employment rowkey 20200630 source

  // winhc_eci_dev company_land_publicity rowkey 20200717 title,location,use_for
  // winhc_eci_dev company_land_announcement rowkey 20200717 e_number,project_name

  // winhc_eci_dev company_bid_list rowkey 20200717 title
  // winhc_eci_dev company_land_transfer rowkey 20200717 num,location
  // winhc_eci_dev company_abnormal_info rowkey 20200717 remove_reason

  // winhc_eci_dev company_own_tax rowkey 20200729 tax_balance,tax_category,tax_num


  //winhc_eci_dev company_equity_info id 20200730 reg_number false


  // winhc_eci_dev company cid 20200630 legal_entity_id,reg_location,business_scope,reg_status,reg_capital,emails,phones


  private val startArgs = Seq(
    Args(tableName = "company_tm", primaryFields = "status_new")
    , Args(tableName = "company_patent_list", primaryFields = "lprs")
    , Args(tableName = "company_copyright_works_list", primaryFields = "type")
    , Args(tableName = "company_copyright_reg_list", primaryFields = "version")
    , Args(tableName = "company_land_publicity", primaryFields = "title,location,use_for")
    , Args(tableName = "company_land_announcement", primaryFields = "e_number,project_name")
    , Args(tableName = "company_bid_list", primaryFields = "title")
    , Args(tableName = "company_land_transfer", primaryFields = "num,location")
    , Args(tableName = "company_employment", primaryFields = "source")
    , Args(tableName = "company_env_punishment", primaryFields = "punish_number")
    , Args(tableName = "company_icp", primaryFields = "domain")

    , Args(tableName = "company_certificate", primaryFields = "type")
    , Args(tableName = "company_abnormal_info", primaryFields = "remove_reason")

    , Args(tableName = "company_own_tax", primaryFields = "tax_balance,tax_category,tax_num")

    , Args(tableName = "company_equity_info", primaryKey = "id", primaryFields = "reg_number", isCopy = false)
    //    , Args(tableName = "company_staff", primaryFields = "staff_type")

  )

  private case class Args(project: String = "winhc_eci_dev"
                          , tableName: String
                          , primaryKey: String = "rowkey"
                          , primaryFields: String
                          , isCopy: Boolean = true)

  def main(args: Array[String]): Unit = {
    if (args.length == 2) {
      val Array(tableName, inc_ds) = args

      val e = startArgs.filter(_.tableName.equals(tableName)).head
      val config = EsConfig.getEsConfigMap ++ mutable.Map(
        "spark.hadoop.odps.project.name" -> e.project,
        "spark.hadoop.odps.spark.local.partition.amt" -> "10"
      )
      val spark = SparkUtils.InitEnv("ChangeExtract", config)

      ChangeExtractHandle(spark, e.project, tableName, e.primaryKey, inc_ds, e.primaryFields.split(",")).calc(e.isCopy)
      spark.stop()
    } else {
      val ds = args(0)
      val project = "winhc_eci_dev"
      val config = EsConfig.getEsConfigMap ++ mutable.Map(
        "spark.hadoop.odps.project.name" -> project,
        "spark.hadoop.odps.spark.local.partition.amt" -> "10"
      )
      val spark = SparkUtils.InitEnv("ChangeExtract", config)

      startArgs.foreach(e => {
        ChangeExtractHandle(spark, e.project, e.tableName, e.primaryKey, ds, e.primaryFields.split(",")).calc(e.isCopy)
      })


      /* val rows =
         """winhc_eci_dev company_tm rowkey 20200717 status_new
           |winhc_eci_dev company_patent_list rowkey 20200717 lprs
           |winhc_eci_dev company_copyright_works_list rowkey 20200717 type
           |winhc_eci_dev company_copyright_reg_list rowkey 20200717 version
           |winhc_eci_dev company_land_publicity rowkey 20200717 title,location,use_for
           |winhc_eci_dev company_land_announcement rowkey 20200717 e_number,project_name
           |winhc_eci_dev company_bid_list rowkey 20200717 title
           |winhc_eci_dev company_land_transfer rowkey 20200717 num,location
           |winhc_eci_dev company_employment rowkey 20200717 source
           |winhc_eci_dev company_env_punishment rowkey 20200717 punish_number
           |winhc_eci_dev company_icp rowkey 20200717 domain
           |""".stripMargin.replace("20200717", ds)
       for (r <- rows.split("\r\n")) {
         if (StringUtils.isNotEmpty(r)) {
           val as = r.split(" ")
           val Array(tmp, tableName, rowkey, inc_ds, pf, isCopy) = if (as.length == 6) as else as :+ "true"
           ChangeExtractHandle(spark, project, tableName, rowkey, inc_ds, pf.split(",")).calc(isCopy.toBoolean)
         }
       }*/
      spark.stop()
    }
  }
}
