package com.winhc.bigdata.spark.jobs.dynamic

/**
 * @Author: XuJiakai
 * @Date: 2020/7/27 17:05
 * @Description:
 */
trait CompanyDynamicHandle {

  private val table_2_sub_info_type_map = Map(
    "CompanyDynamicHandleTest" -> "MyTest"
    , "" -> "eci_detail" //工商信息
    , "company_land_publicity" -> "land_publicity" //土地公示
    , "company_land_announcement" -> "land_announcement" //土地购买
    , "company_land_transfer" -> "land_transfer" //土地转让
    , "company_land_mortgage" -> "land_mortgage" //土地抵押
    , "company_bid_list" -> "tender_es" //中标信息ES
    , "" -> "enterprise_shixin" //失信
    , "" -> "enterprise_zhixing" //被执
    , "" -> "shareholder_shixin" //股东失信
    , "" -> "shareholder_zhixing" //股东被执
    , "" -> "tender_qichacha" //中标信息企查查
    , "company_abnormal_info" -> "eci_exception" //经营异常
    , "" -> "eci_zscq" //知识产权
    , "" -> "eci_wenshu" //裁判文书
    , "" -> "court_announcement" //法院公告
    , "" -> "" //对外投资
    , "company_punishment_info" -> "punishment_info" //行政处罚
    , "company_punishment_info_creditchina" -> "punishment_info_creditchina" //行政处罚-信用中国
    , "" -> "eci_chattel" //动产抵押
    , "company_env_punishment" -> "env_punishment" //环保处罚
    , "" -> "judicial_assistance" //股权冻结
    , "" -> "publish_notice" //公示催告
    , "" -> "serious_violation" //严重违法
    , "" -> "simple_cancellation" //简易注销
    , "company_equity_info" -> "stock_pledge" //股权出质
    , "" -> "tax_illegal" //税收违法
    , "" -> "tax_owenotice" //欠税公告
    , "" -> "judicial" //司法拍卖
    , "" -> "recruit" //招聘信息
    , "" -> "liquidation_information" //清算信息
    , "" -> "investor_equity_change" //大股东变更
    , "" -> "actual_controller_change" //实际控制人变更
    , "" -> "court_notice" //开庭公告
  )

  private val table_2_info_type = Map(
    "CompanyDynamicHandleTest" -> "0"
    , "" -> "1" //工商信息
    , "" -> "2" // 企业失信被执
    , "" -> "3" // 企业股东失信被执
    , "company_abnormal_info" -> "4" // 经营异常
    , "" -> "5" // 知识产权
    , "" -> "6" // 裁判文书
    , "" -> "7" // 法院公告
    , "" -> "8" // 对外投资
    , "" -> "9" // 动产抵押
    , "" -> "10" // 司法拍卖
    , "company_land_publicity" -> "11-1" // 土地信息-地块公示
    , "company_land_announcement" -> "11-2" // 土地信息-购地信息
    , "company_land_transfer" -> "11-3" // 土地信息-土地转让
    , "company_land_mortgage" -> "11-4" // 土地信息-土地抵押
    , "company_bid_list" -> "12" // 中标信息
    , "" -> "13" // 招聘信息
    , "company_punishment_info" -> "14-1" // 行政处罚
    , "company_punishment_info_creditchina" -> "14-2" // 行政处罚-信用中国
    , "" -> "15" // 公示催告
    , "company_env_punishment" -> "16" // 环保处罚
    , "company_equity_info" -> "17" // 股权出质
    , "" -> "18" // 严重违法
    , "" -> "19" // 简易注销
    , "" -> "20" // 欠税公告
    , "" -> "21" // 税收违法
    , "" -> "22" // 股权冻结
    , "" -> "23" // 清算信息
    , "" -> "24" // 大股东变更
    , "" -> "25" // 实际控制人变更
    , "" -> "26" // 开庭公告
    , "" -> "27" // 新闻信息
    , "" -> "28" // 股东信息
    , "" -> "29" // 最终受益人
    , "" -> "30" // 主要成员
    , "" -> "31" // 融资动态
    , "" -> "32" // 企业公告
    , "" -> "33" // 抽查检查
    , "" -> "34" // 行政许可
    , "" -> "35" // 双随机抽查
    , "" -> "36" // 限制高消费
    , "" -> "37" // 被执行人
    , "" -> "38" // 送达报告
  )

  /**
   *
   * @param rowkey
   * @param cid
   * @param change_fields
   * @param old_map
   * @param new_map
   * @return cid
   *         cname
   *         info_type
   *         rta_desc
   *         change_content
   *         change_time
   *         biz_id
   *         sub_info_type
   *         info_risk_level
   *         winhc_suggest
   */
  def handle(rowkey: String, bizDate: String, cid: String, change_fields: Seq[String], old_map: Map[String, String], new_map: Map[String, String], cname: String = null, suggestion: String = null): Seq[(String, String, String, String, String, String, String, String, String, String)] = {
    Seq((cid
      , cname
      , get_info_type()
      , get_rta_desc(old_map, new_map)
      , get_change_content(old_map, new_map)
      , get_change_time(bizDate, new_map)
      , get_biz_id(rowkey)
      , get_sub_info_type()
      , get_info_risk_level(old_map, new_map)
      , if (suggestion == null) "被监控企业流动资金紧张，可能存在经营困难的情况。建议立即与被监控企业书面对账，适当催促其履行债务并持续监控。" else suggestion
    ))
  }

  /**
   * 来源表的变更类型，insert or update
   *
   * @return
   */
  def org_type(): Seq[String] = Seq("insert")


  /**
   * 信息描述
   *
   * @param old_map
   * @param new_map
   * @return
   */
  protected def get_rta_desc(old_map: Map[String, String], new_map: Map[String, String]): String

  /**
   * 信息类型，大类
   *
   * @return
   */
  protected def get_info_type(): String = table_2_info_type(getClass.getSimpleName)

  /**
   * 变更内容
   *
   * @param old_map
   * @param new_map
   * @return
   */
  protected def get_change_content(old_map: Map[String, String], new_map: Map[String, String], cname: String = null): String

  /**
   * 变更时间
   *
   * @param new_map
   * @return
   */
  protected def get_change_time(bizDate: String, new_map: Map[String, String]): String = bizDate

  /**
   * 业务id
   *
   * @param rowkey
   * @return
   */
  protected def get_biz_id(rowkey: String): String = rowkey

  /**
   * 子信息类型，小类
   *
   * @return
   */
  protected def get_sub_info_type(): String = table_2_sub_info_type_map(getClass.getSimpleName)

  /**
   * 风险等级
   *
   * @param old_map
   * @param new_map
   * @return
   */
  protected def get_info_risk_level(old_map: Map[String, String], new_map: Map[String, String]): String

}

