package com.winhc.bigdata.spark.jobs.dynamic.tables

import com.winhc.bigdata.spark.jobs.dynamic.CompanyDynamicHandle

/**
 * @Author yyn
 * @Date 2020/8/4
 * @Description TODO
 */
//土地抵押
case class company_land_mortgage() extends CompanyDynamicHandle {
  /**
   * 信息描述
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override def get_rta_desc(old_map: Map[String, String], new_map: Map[String, String]): String = new_map("land_num")

  /**
   * 变更内容
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override def get_change_content(old_map: Map[String, String], new_map: Map[String, String], cname: String = null): String = {
    s"""抵押土地评估金额:$new_map("evaluate_amount")万元\n
       |抵押金额：$new_map("mortgage_amount")万元\n
       |抵押开始时间：$new_map("start_date")\n
       |抵押结束时间：$new_map("end_date")\n""".stripMargin
  }

  /**
   * 变更时间
   *
   * @param new_map
   * @return
   */
//  override def get_change_time(new_map: Map[String, String]): String = new_map("biz_date")

  /**
   * 风险等级
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override def get_info_risk_level(old_map: Map[String, String], new_map: Map[String, String]): String = {
    //获取抵押类型：mortgagee：只有抵押权人；mortgagor：只有抵押人；bothsame：同为抵押人和抵押权人（有这样的数据）；bothone：两者数据均有的抵押人；bothtwo：两者数据均有的抵押权人
    val mortType = new_map("type")
    if (mortType != null && !mortType.isEmpty) {
      //抵押权人
      if(mortType.equals("mortgagee") || mortType.equals("bothtwo")) {
        "提示信息"
      }
      else{//抵押人
        "警示信息"
      }
    }
    else{
      null//脏数据
    }
  }
  /**
   *
   * @param rowkey
   * @param cid
   * @param change_fields
   * @param old_map
   * @param new_map
   * @return cid
   *         cname
   *         info_type
   *         rta_desc
   *         change_content
   *         change_time
   *         biz_id
   *         sub_info_type
   *         info_risk_level
   *         winhc_suggest
   */
  override def handle(rowkey: String, bizDate: String, cid: String, change_fields: Seq[String], old_map: Map[String, String], new_map: Map[String, String], cname: String, suggestion: String): Seq[(String, String, String, String, String, String, String, String, String, String)] = {
//获取抵押类型：mortgagee：只有抵押权人；mortgagor：只有抵押人；bothsame：同为抵押人和抵押权人（有这样的数据）；bothone：两者数据均有的抵押人；bothtwo：两者数据均有的抵押权人
    val mortType = new_map("type")
    if (mortType != null && !mortType.isEmpty) {
      //抵押权人
      if(mortType.equals("mortgagee") || mortType.equals("bothtwo")) {
        super.handle(rowkey, bizDate, cid, change_fields, old_map, new_map, cname, "该企业有接受其他企业土地抵押行为")
      }
      else{//抵押人
        super.handle(rowkey, bizDate, cid, change_fields, old_map, new_map, cname, "该企业有土地抵押行为")
      }
    }
    else{
      null//脏数据
    }
  }
}