package com.winhc.bigdata.spark.udf

import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types._

/**
 * @Author: XuJiakai
 * @Date: 2020/8/3 14:52
 * @Description:
 */
class CompanyStaffAggs(max: Int = 500) extends UserDefinedAggregateFunction {
  override def inputSchema: StructType = StructType(StructField("types", StringType) :: Nil)

  override def bufferSchema: StructType = StructType(Array(StructField("buffer", ArrayType(StringType, containsNull = false))))

  override def dataType: DataType = StringType

  override def deterministic: Boolean = true

  override def initialize(buffer: MutableAggregationBuffer): Unit = buffer.update(0, Seq.empty[String])

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {
    val v = input.getString(0)
    if (StringUtils.isNotEmpty(v)) {
      buffer(0) = v.split(",") ++ buffer.getSeq[String](0)
    }
  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {
    buffer1(0) = buffer1.getSeq[String](0) ++ buffer2.getSeq[String](0)
  }

  //.diff(Seq("未知", "未公开"))
  override def evaluate(buffer: Row): Any = buffer.getSeq[String](0).distinct.mkString(",")
}
