package com.winhc.bigdata.spark.utils

import com.winhc.bigdata.spark.config.HBaseConfig
import org.apache.hadoop.hbase.client.Put
import org.apache.hadoop.hbase.io.ImmutableBytesWritable
import org.apache.hadoop.hbase.util.Bytes
import org.apache.spark.sql.SparkSession
import org.apache.spark.sql.functions.col

import scala.annotation.meta.getter


/**
 * @Author: XuJiakai
 * @Date: 2020/6/22 16:33
 * @Description:
 */
case class CompanyIncSummary(s: SparkSession,
                             project: String, //表所在工程名
                             tableName: String, //主表名（不加前辍）
                             cidField: String, // 公司id fieldName
                             dupliCols: Seq[String] //去重主键
                            ) extends LoggingUtils {
  @(transient@getter) val spark: SparkSession = s
  private val f_bytes: Array[Byte] = Bytes.toBytes("F")
  private val name_bytes: Array[Byte] = Bytes.toBytes(tableName.toUpperCase)

  def calc(): Unit = {
    val ads_table = s"${project}.ads_$tableName" //存量ads表
    val inc_ads_table = s"${project}.inc_ads_$tableName"

    val partition = sql(s"show partitions $ads_table").collect.toList
      .map(_.getString(0).split("=")(1))
      .last


    val ads_table_cols = spark.table(ads_table).columns.filter(l => {
      !l.equals("ds") && !l.equals("rowkey") && !l.equals("flag") && !l.equals("new_cids") && !l.equals("cids") && !l.equals("cid") && !l.equals("new_litigant_cids")
    }).toList.sorted


    val inc_ads_table_cols = spark.table(inc_ads_table).columns.filter(l => {
      !l.equals("ds") && !l.equals("rowkey") && !l.equals("flag") && !l.equals("new_cids") && !l.equals("cids") && !l.equals("cid") && !l.equals("new_litigant_cids")
    }).toList.sorted


    val new_cols = ads_table_cols.intersect(inc_ads_table_cols)
    if (new_cols.size != inc_ads_table_cols.size || new_cols.size != ads_table_cols.size) {
      println(ads_table_cols.sorted)
      println(inc_ads_table_cols.sorted)
      println("cols not equals!")
      println(new_cols.sorted)
    }

    sql(
      s"""
         |SELECT  ${new_cols.map(getCastCols(_, "")).mkString(",")}
         |FROM    (
         |            SELECT  tmp.*
         |                    ,ROW_NUMBER() OVER(PARTITION BY ${dupliCols.mkString(",")} ORDER BY update_time DESC ) c
         |            FROM    (
         |                        SELECT  ${new_cols.map(getCastCols(_, "org_tab.")).mkString(",")}
         |                        FROM    (
         |                                    SELECT  DISTINCT $cidField as $cidField
         |                                    FROM    $inc_ads_table
         |                                    WHERE   ds > $partition
         |                                ) id_table
         |                        JOIN (
         |                                      SELECT  *
         |                                      FROM    $ads_table
         |                                      WHERE   ds = '$partition'
         |                                  ) org_tab
         |                        ON      id_table.$cidField = org_tab.$cidField
         |                        UNION ALL
         |                        SELECT  ${new_cols.map(getCastCols(_, "")).mkString(",")}
         |                        FROM    $inc_ads_table
         |                        WHERE   ds > $partition
         |                    ) AS tmp
         |        ) tmp2
         |WHERE   tmp2.c = 1
         |""".stripMargin).cache().createOrReplaceTempView("inc_tmp_view")


    val jobConf = HBaseConfig.HBaseOutputJobConf("COMPANY_SUMMARY")
    sql(
      s"""
         |SELECT  ${cidField} as cid
         |        ,COUNT(1) as num
         |FROM    inc_tmp_view
         |GROUP BY $cidField
         |""".stripMargin)
      .select(Seq("cid", "num").map(column => col(column).cast("string")): _*)
      .rdd
      .filter(r => {
        r.get(1) != null && !"0".equals(r.getString(1))
      }).map(row => {
      val id = row.getString(0)
      val num = row.getString(1)
      val put = new Put(Bytes.toBytes(id))
      put.addColumn(f_bytes, name_bytes, Bytes.toBytes(num))
      (new ImmutableBytesWritable, put)
    }).filter(_ != null).saveAsHadoopDataset(jobConf)
  }

  private def getCastCols(name: String, pre: String): String = {
    val list = List("cid", "new_cid", "ncid")
    if (list.contains(name)) {
      return s"CAST(${pre}${name} as BIGINT) $name"
    }
    pre + name
  }
}
